@@ -23,11 +23,10 @@ import java.math.BigInteger;
 
 import org.apache.parquet.column.ColumnDescriptor;
 import org.apache.spark.sql.catalyst.InternalRow;
+import org.apache.spark.sql.catalyst.util.ResolveDefaultColumns;
 import org.apache.spark.sql.types.*;
 import org.apache.spark.unsafe.types.UTF8String;
 
-import org.apache.comet.shims.ShimResolveDefaultColumns;
-
 /**
  * A column reader that always return constant vectors. Used for reading partition columns, for
  * instance.
@@ -39,24 +38,37 @@ public class ConstantColumnReader extends MetadataColumnReader {
   /** The constant value in the format of Object that are used to initialize this column reader. */
   private Object value;
 
-  public ConstantColumnReader(StructField field, int batchSize, boolean useDecimal128) {
+  ConstantColumnReader(StructField field, int batchSize, boolean useDecimal128) {
     this(field.dataType(), TypeUtil.convertToParquet(field), batchSize, useDecimal128);
-    this.value = ShimResolveDefaultColumns.getExistenceDefaultValue(field);
+    this.value =
+        ResolveDefaultColumns.getExistenceDefaultValues(new StructType(new StructField[] {field}))[
+            0];
     init(value);
   }
 
-  public ConstantColumnReader(
+  ConstantColumnReader(
       StructField field, int batchSize, InternalRow values, int index, boolean useDecimal128) {
     this(field.dataType(), TypeUtil.convertToParquet(field), batchSize, useDecimal128);
     init(values, index);
   }
 
+  /**
+   * @deprecated since 0.10.0, will be removed in 0.11.0.
+   * @see <a href="https://github.com/apache/datafusion-comet/issues/2079">Comet Issue #2079</a>
+   */
   public ConstantColumnReader(
       DataType type, ColumnDescriptor descriptor, Object value, boolean useDecimal128) {
     super(type, descriptor, useDecimal128, true);
     this.value = value;
   }
 
+  // Used by Iceberg
+  public ConstantColumnReader(
+      DataType type, ParquetColumnSpec spec, Object value, boolean useDecimal128) {
+    super(type, spec, useDecimal128, true);
+    this.value = value;
+  }
+
   ConstantColumnReader(
       DataType type, ColumnDescriptor descriptor, int batchSize, boolean useDecimal128) {
     super(type, descriptor, useDecimal128, true);
