@@ -33,6 +33,5 @@ object ShimBatchReader {
       -1,
       Array.empty[String],
       0,
-      0,
-      Map.empty)
+      0)
 }
