@@ -21,7 +21,23 @@ import json
 from pyspark.sql import SparkSession
 import time
 
-def main(benchmark: str, data_path: str, query_path: str, iterations: int, output: str, name: str, query_num: int = None):
+# rename same columns aliases
+# a, a, b, b -> a, a_1, b, b_1
+#
+# Important for writing data where column name uniqueness is required
+def dedup_columns(df):
+    counts = {}
+    new_cols = []
+    for c in df.columns:
+        if c not in counts:
+            counts[c] = 0
+            new_cols.append(c)
+        else:
+            counts[c] += 1
+            new_cols.append(f"{c}_{counts[c]}")
+    return df.toDF(*new_cols)
+
+def main(benchmark: str, data_path: str, query_path: str, iterations: int, output: str, name: str, query_num: int = None, write_path: str = None):
 
     # Initialize a SparkSession
     spark = SparkSession.builder \
@@ -88,11 +104,25 @@ def main(benchmark: str, data_path: str, query_path: str, iterations: int, outpu
                     if len(sql) > 0:
                         print(f"Executing: {sql}")
                         df = spark.sql(sql)
-                        df.explain()
-                        rows = df.collect()
-                        df.explain()
+                        df.explain("formatted")
 
-                        print(f"Query {query} returned {len(rows)} rows")
+                        if write_path is not None:
+                            # skip results with empty schema
+                            # coming across for running DDL stmt
+                            if len(df.columns) > 0:
+                                output_path = f"{write_path}/q{query}"
+                                # rename same column names for output
+                                # a, a, b, b => a, a_1, b, b_1
+                                # output doesn't allow non unique column names
+                                deduped = dedup_columns(df)
+                                # sort by all columns to have predictable output dataset for comparison
+                                deduped.orderBy(*deduped.columns).coalesce(1).write.mode("overwrite").parquet(output_path)
+                                print(f"Query {query} results written to {output_path}")
+                            else:
+                                print(f"Skipping write: DataFrame has no schema for {output_path}")
+                        else:
+                            rows = df.collect()
+                            print(f"Query {query} returned {len(rows)} rows")
 
                 end_time = time.time()
                 print(f"Query {query} took {end_time - start_time} seconds")
@@ -123,6 +153,8 @@ if __name__ == "__main__":
     parser.add_argument("--output", required=True, help="Path to write output")
     parser.add_argument("--name", required=True, help="Prefix for result file e.g. spark/comet/gluten")
     parser.add_argument("--query", required=False, type=int, help="Specific query number to run (1-based). If not specified, all queries will be run.")
+    parser.add_argument("--write", required=False, help="Path to save query results to, in Parquet format.")
     args = parser.parse_args()
 
-    main(args.benchmark, args.data, args.queries, int(args.iterations), args.output, args.name, args.query)
\ No newline at end of file
+    main(args.benchmark, args.data, args.queries, int(args.iterations), args.output, args.name, args.query, args.write)
+
