@@ -15,7 +15,6 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use crate::execution::operators::CopyExec;
 use arrow::datatypes::SchemaRef;
 use datafusion::physical_plan::ExecutionPlan;
 use std::sync::Arc;
@@ -43,15 +42,11 @@ impl SparkPlan {
         native_plan: Arc<dyn ExecutionPlan>,
         children: Vec<Arc<SparkPlan>>,
     ) -> Self {
-        let mut additional_native_plans: Vec<Arc<dyn ExecutionPlan>> = vec![];
-        for child in &children {
-            collect_additional_plans(Arc::clone(&child.native_plan), &mut additional_native_plans);
-        }
         Self {
             plan_id,
             native_plan,
             children,
-            additional_native_plans,
+            additional_native_plans: vec![],
         }
     }
 
@@ -66,9 +61,6 @@ impl SparkPlan {
         for plan in &additional_native_plans {
             accum.push(Arc::clone(plan));
         }
-        for child in &children {
-            collect_additional_plans(Arc::clone(&child.native_plan), &mut accum);
-        }
         Self {
             plan_id,
             native_plan,
@@ -87,12 +79,3 @@ impl SparkPlan {
         &self.children
     }
 }
-
-fn collect_additional_plans(
-    child: Arc<dyn ExecutionPlan>,
-    additional_native_plans: &mut Vec<Arc<dyn ExecutionPlan>>,
-) {
-    if child.as_any().is::<CopyExec>() {
-        additional_native_plans.push(Arc::clone(&child));
-    }
-}
