@@ -36,7 +36,7 @@ use datafusion::physical_expr::expressions::StatsType;
 /// we have our own implementation is that DataFusion has UInt64 for state_field `count`,
 /// while Spark has Double for count. Also we have added `null_on_divide_by_zero`
 /// to be consistent with Spark's implementation.
-#[derive(Debug)]
+#[derive(Debug, PartialEq, Eq)]
 pub struct Stddev {
     name: String,
     signature: Signature,
@@ -44,6 +44,15 @@ pub struct Stddev {
     null_on_divide_by_zero: bool,
 }
 
+impl std::hash::Hash for Stddev {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.name.hash(state);
+        self.signature.hash(state);
+        (self.stats_type as u8).hash(state);
+        self.null_on_divide_by_zero.hash(state);
+    }
+}
+
 impl Stddev {
     /// Create a new STDDEV aggregate function
     pub fn new(
