@@ -15,12 +15,10 @@
 // specific language governing permissions and limitations
 // under the License.
 
-mod date_arithmetic;
 mod date_trunc;
 mod extract_date_part;
 mod timestamp_trunc;
 
-pub use date_arithmetic::{spark_date_add, spark_date_sub};
 pub use date_trunc::SparkDateTrunc;
 pub use extract_date_part::SparkHour;
 pub use extract_date_part::SparkMinute;
