@@ -20,11 +20,13 @@
 package org.apache.comet.rules
 
 import org.apache.spark.sql.catalyst.optimizer.{BuildLeft, BuildRight, BuildSide, JoinSelectionHelper}
-import org.apache.spark.sql.catalyst.plans.LeftAnti
+import org.apache.spark.sql.catalyst.plans.{LeftAnti, LeftSemi}
 import org.apache.spark.sql.catalyst.plans.logical.Join
 import org.apache.spark.sql.execution.{SortExec, SparkPlan}
 import org.apache.spark.sql.execution.joins.{ShuffledHashJoinExec, SortMergeJoinExec}
 
+import org.apache.comet.CometSparkSessionExtensions.withInfo
+
 /**
  * Adapted from equivalent rule in Apache Gluten.
  *
@@ -65,8 +67,13 @@ object RewriteJoin extends JoinSelectionHelper {
   def rewrite(plan: SparkPlan): SparkPlan = plan match {
     case smj: SortMergeJoinExec =>
       getSmjBuildSide(smj) match {
-        case Some(BuildRight) if smj.joinType == LeftAnti =>
-          // https://github.com/apache/datafusion-comet/issues/457
+        case Some(BuildRight) if smj.joinType == LeftAnti || smj.joinType == LeftSemi =>
+          // LeftAnti https://github.com/apache/datafusion-comet/issues/457
+          // LeftSemi https://github.com/apache/datafusion-comet/issues/2667
+          withInfo(
+            smj,
+            "Cannot rewrite SortMergeJoin to HashJoin: " +
+              s"BuildRight with ${smj.joinType} is not supported")
           plan
         case Some(buildSide) =>
           ShuffledHashJoinExec(
