@@ -22,8 +22,8 @@ package org.apache.spark.sql.comet
 import java.util.UUID
 import java.util.concurrent.{Future, TimeoutException, TimeUnit}
 
-import scala.collection.JavaConverters._
 import scala.concurrent.Promise
+import scala.jdk.CollectionConverters._
 import scala.util.control.NonFatal
 
 import org.apache.spark.{broadcast, SparkException}
@@ -53,6 +53,7 @@ import org.apache.comet.vector.CometPlainVector
  */
 case class CometColumnarToRowExec(child: SparkPlan)
     extends ColumnarToRowTransition
+    with CometPlan
     with CodegenSupport {
   // supportsColumnar requires to be only called on driver side, see also SPARK-37779.
   assert(Utils.isInRunningSparkTask || child.supportsColumnar)
