@@ -21,6 +21,4 @@ package org.apache.spark.sql.comet.shims
 
 import org.apache.spark.shuffle.ShuffleWriteProcessor
 
-trait ShimCometShuffleWriteProcessor extends ShuffleWriteProcessor {
-
-}
+trait ShimCometShuffleWriteProcessor extends ShuffleWriteProcessor {}
