@@ -21,4 +21,7 @@ package org.apache.comet.shims
 
 import org.apache.spark.sql.SQLQueryTestHelper
 
-trait ShimCometTPCHQuerySuite extends SQLQueryTestHelper {}
+trait ShimCometTPCHQuerySuite extends SQLQueryTestHelper {
+  // no need to define getNormalizedQueryExecutionResult in this version of the shim because it exists
+  // in SQLQueryTestHelper in Spark 3.5
+}
